<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2006 m3 Media Services Limited                         |
| For contact details, see: http://www.m3.net/                              |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: Delivery.php 4722 2006-04-21 17:08:16Z andrew@m3.net $
*/

/**
 * @package    MaxDelivery
 * @author     Scott Switzer <scott@switzer.org>
 *
 * A file containing common functions for the Max delivery engine.
 */

/**
 * Remove an assortment of special characters from a variable or array:
 * 1.  Strip slashes if magic quotes are turned on.
 * 2.  Strip out any HTML
 * 3.  Strip out any CRLF
 * 4.  Remove any white space
 *
 * @access  public
 * @param   string $var  The variable to process.
 * @return  string       $var, minus any special quotes.
 */
function removeSpecialChars(&$var)
{
    static $magicQuotes;
    if (!isset($magicQuotes)) {
        $magicQuotes = get_magic_quotes_gpc();
    }
    if (isset($var)) {
        if (!is_array($var)) {
            if ($magicQuotes) {
                $var = stripslashes($var);
            }
            $var = strip_tags($var);
            $var = str_replace(array("\n", "\r"), array('', ''), $var);
            $var = trim($var);
        } else {
            array_walk($var, 'removeSpecialChars');
        }
    }
}

/**
 * Generate a unique ID (used for the ID for unique users)
 *
 * @return string Unique MD5 string
 */
function generateUniqueId()
{
    return md5(uniqid(mt_rand(), true));
}

/**
 * Start benchmarking, if it is turned on
 *
 * @param array $conf The configuration array
 * @param object $timer The timer object from Pear
 */
function startBenchmark()
{
    $conf = $GLOBALS['_MAX']['CONF'];
    if ($conf['debug']['benchmark']) {
        include_once 'Benchmark/Timer.php';
        $timer = new Benchmark_Timer();
        $timer->start();
        $GLOBALS['timer'] = $timer;
    }
}

/**
 * Stop benchmarking, if it is turned on
 *
 * @param array $conf The configuration array
 * @param object $timer The timer object from Pear
 */
function stopBenchmark()
{
    $conf = $GLOBALS['_MAX']['CONF'];
    if ($conf['debug']['benchmark']) {
        require_once MAX_PATH . '/lib/max/Delivery/log.php';
        $timer = $GLOBALS['timer'];
        $timer->stop();
        MAX_logBenchmark(basename($_SERVER['PHP_SELF']), $_SERVER['QUERY_STRING'], $timer->timeElapsed());
    }
}

/**
 * Get the amount of bytes of memory usage at this particular moment.
 *
 * @return int The total number of bytes that this PHP process is currently using.
 */
function getMemoryUsage()
{
    if (function_exists('memory_get_usage')) {
        return memory_get_usage();
    } elseif ( strpos( strtolower($_SERVER["OS"]), 'windows') !== false) {
        // Windows workaround
        $output = array();
        exec('tasklist /FI "PID eq ' . getmypid() . '" /FO LIST', $output);
        return substr($output[5], strpos($output[5], ':') + 1);
    } else {
        return '<b style="color: red;">no value</b>';
    }
}

/**
 * A function to generate the URL for delivery scripts.
 *
 * @param string $file The file name of the delivery script.
 * @return string The URL to the delivery script.
 */
function constructDeliveryUrl($file)
{
        $conf = $GLOBALS['_MAX']['CONF'];
        return 'http://' . $conf['webpath']['delivery'] . '/' . $file;
}

/**
 * A function to generate the secure URL for delivery scripts.
 *
 * @param string $file The file name of the delivery script.
 * @return string The URL to the delivery script.
 */
function constructSecureDeliveryUrl($file)
{
        $conf = $GLOBALS['_MAX']['CONF'];
        if ($conf['max']['sslPort'] != 443) {
            // Fix the delivery host
            $path = preg_replace('#/#', ':' . $conf['max']['sslPort'] . '/', $conf['webpath']['deliverySSL']);
        } else {
            $path = $conf['webpath']['deliverySSL'];
        }
        return 'https://' . $path . '/' . $file;
}

/**
 * A function to generate the URL for delivery scripts without a protocol.
 *
 * @param string $file The file name of the delivery script.
 * @param boolean $ssl Use the SSL delivery path (true) or not. Default is false.
 * @return string The parital URL to the delivery script (i.e. without
 *                an 'http:' or 'https:' prefix).
 */
function constructPartialDeliveryUrl($file, $ssl = false)
{
        $conf = $GLOBALS['_MAX']['CONF'];
        if ($ssl) {
            return '//' . $conf['webpath']['deliverySSL'] . '/' . $file;
        } else {
            return '//' . $conf['webpath']['delivery'] . '/' . $file;
        }
}

?>